import Resource from '../src'
import { DefaultClient } from '../src/client'
import RelatedManager from '../src/related'
import { normalizerFactory, NumberNormalizer } from '../src/helpers/normalization'

class CustomRelatedManager<T extends typeof Resource> extends RelatedManager<T> {
    batchSize = 20
}

class CustomClient extends DefaultClient {
    get() {
        console.log('GET', arguments[0], arguments[1] || {})
        // @ts-ignore
        return DefaultClient.prototype.get.apply(this, arguments)
    }

    patch() {
        console.log('PATCH', arguments[0], arguments[1] || {})
        // @ts-ignore
        return DefaultClient.prototype.patch.apply(this, arguments)
    }
}

class BaseResource extends Resource {
    static client = new CustomClient('http://localhost:8099')
    static RelatedManagerClass = CustomRelatedManager
}

class UserResource extends BaseResource {
    static endpoint = '/users'
    static validation = {
        phone(value: any, instance: UserResource, ValidationError: any) {
            let regExp = /^[\+]?[(]?[0-9]{3}[)]?[-\s\.]?[0-9]{3}[-\s\.]?[0-9]{4,6}$/im
            if (!regExp.test(value)) {
                throw new ValidationError('phone')
            }
        },
    }
}

class TodoResource extends BaseResource {
    static endpoint = '/todos'
    static related = {
        user: UserResource,
    }
}

class PostResource extends BaseResource {
    static endpoint = '/posts'
    static related = {
        user: UserResource,
    }
}

class GroupResource extends BaseResource {
    static endpoint = '/groups'
    static related = {
        users: UserResource,
        owner: UserResource,
        todos: TodoResource,
    }

    static normalization = {
        owner: new NumberNormalizer(),
    }
}

;(async function() {
    class PostResource extends BaseResource {
        static endpoint = '/posts'

        static related = {
            user: UserResource,
        }

        static normalization = {
            user: new NumberNormalizer(),
        }
    }

    class GroupResource extends BaseResource {
        static endpoint = '/groups'
        static related = {
            user: UserResource,
            owner: UserResource,
            post: PostResource,
        }

        static normalization = {
            owner: new NumberNormalizer(),
        }
    }

    let post = await PostResource.detail(3)
    console.log(post.attributes.user)
    // => 123

    let comment = await GroupResource.detail('1')
    console.log(comment.attributes.owner)
})()
